<?php

namespace App\Service;

use App\Entity\Article;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use App\Repository\ArticleRepository;
use Debril\RssAtomBundle\Provider\FeedProviderInterface;
use FeedIo\Feed;
use FeedIo\Feed\Item;
use FeedIo\Feed\Item\Author;
use Symfony\Component\HttpFoundation\Request;
use FeedIo\FeedInterface;
use Symfony\Component\Routing\RouterInterface;

class BlogFeedProvider implements FeedProviderInterface
{

    public function __construct(
        protected ArticleRepository $articleRepository,
        protected UrlGeneratorInterface $router,
        protected MarkdownParser $markdownParser
    )
    {

    }

    public function getFeed(Request $request): FeedInterface
    {
        $feed = new Feed();
        $feed->setTitle('Opinion sur le réel');
        $feed->setLink($this->router->generate('app_index', [], RouterInterface::ABSOLUTE_URL));
        $feed->setLanguage('fr');
        foreach($this->getItems() as $item) {
            $feed->add($item);
        }
        return $feed;
    }

    public function getItems(): \Iterator
    {
        $articles = $this->articleRepository->findBy(criteria: ['state' => 'published'], orderBy: ['publicationDate' => 'DESC']);
        foreach($articles as $article) {
            /** @var Article $article */
            $item = new Item();
            $item
                ->setContent($this->markdownParser->convertToHtml($article->getContent()))
                ->setTitle($article->getTitle())
                ->setAuthor((new Author())->setName($article->getAuthor()->getPseudo()))
                ->setLastModified(\DateTime::createFromImmutable($article->getPublicationDate()))
                ->setLink($this->router->generate('app_view', ['slug' => $article->getSlug()], RouterInterface::ABSOLUTE_URL))
            ;
            yield $item;
        }
    }

    public function getFeedMetadata($feedId)
    {
        return [
            'title' => 'Mon Blog',
            'description' => 'Les derniers articles de mon blog',
            'link' => 'https://example.com/blog',
            'ttl' => 60,
        ];
    }
}
